@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Lite;
 
-our $DATE = '2014-12-20'; # DATE
-our $VERSION = '0.70'; # VERSION
+our $DATE = '2014-12-23'; # DATE
+our $VERSION = '0.71'; # VERSION
 
 use 5.010001;
 # use strict; # already enabled by Mo
@@ -313,164 +313,36 @@ sub run_version {
 }
 
 sub run_help {
+    require Perinci::CmdLine::Help;
+
     my ($self, $r) = @_;
 
     my @help;
     my $scn    = $r->{subcommand_name};
     my $scd    = $r->{subcommand_data};
-    my $meta   = $self->get_meta($r, $scd->{url} // $self->{url});
-    my $args_p = $meta->{args} // {};
 
-    # summary
-    my $cmdname = $self->get_program_and_subcommand_name($r);
-    push @help, $cmdname;
-    {
-        my $sum = ($scd ? $scd->{summary} : undef) //
-            $meta->{summary};
-        last unless $sum;
-        push @help, " - ", $sum, "\n";
-    }
+    # XXX use 'delete local' when we bump minimal perl to 5.12
+    my $common_opts = { %{$self->common_opts} };
+    # hide usage '--subcommands' if we have subcommands but user has specified a
+    # subcommand to use
+    my $has_sc_no_sc = $self->subcommands && !length($r->{subcommand_name});
+    delete $common_opts->{subcommands} if $self->subcommands && !$has_sc_no_sc;
 
-    my $clidocdata;
+    my $meta   = $has_sc_no_sc ? {v=>1.1} :
+        $self->get_meta($r, $scd->{url} // $self->{url});
 
-    # usage
-    push @help, "\n";
-    push @help, "Usage:\n";
-    {
-        # we have subcommands but user has not specified any to choose
-        my $has_sc_no_sc = $self->subcommands && !length($r->{subcommand_name});
+    my $res = Perinci::CmdLine::Help::gen_help(
+        program_name => $self->get_program_and_subcommand_name($r),
+        program_summary => $scd ? $scd->{summary} : undef,
+        program_description => $scd ? $scd->{description} : undef,
+        meta => $meta,
+        common_opts => $self->common_opts,
+        per_arg_json => $self->per_arg_json,
+        per_arg_yaml => $self->per_arg_yaml,
+    );
 
-        push @help, "  $cmdname --help (or -h, -?)\n";
-        push @help, "  $cmdname --version (or -v)\n";
-        push @help, "  $cmdname --subcommands\n" if $has_sc_no_sc;
-
-        require Perinci::Sub::To::CLIDocData;
-        my $res;
-        if ($has_sc_no_sc) {
-            $res = Perinci::Sub::To::CLIDocData::gen_cli_doc_data_from_meta(
-                meta => {v=>1.1}, meta_is_normalized => 1,
-                common_opts  => $self->common_opts,
-                per_arg_json => $self->per_arg_json,
-                per_arg_yaml => $self->per_arg_yaml,
-            );
-        } else {
-            $res = Perinci::Sub::To::CLIDocData::gen_cli_doc_data_from_meta(
-                meta => $meta, meta_is_normalized => 1,
-                common_opts  => $self->common_opts,
-                per_arg_json => $self->per_arg_json,
-                per_arg_yaml => $self->per_arg_yaml,
-            );
-        }
-        die [500, "gen_cli_doc_data_from_meta failed: ".
-                 "$res->[0] - $res->[1]"] unless $res->[0] == 200;
-        $clidocdata = $res->[2];
-        my $usage = $clidocdata->{usage_line};
-        $usage =~ s/\[\[prog\]\]/$cmdname/;
-        push @help, "  $usage\n";
-    }
-
-    # example
-    {
-        last unless @{ $clidocdata->{examples} };
-        push @help, "\n";
-        push @help, "Examples:\n";
-        my $i = 0;
-        my $egs = $clidocdata->{examples};
-        for my $eg (@$egs) {
-            $i++;
-            my $cmdline = $eg->{cmdline};
-            $cmdline =~ s/\[\[prog\]\]/$cmdname/;
-            push @help, "  $eg->{summary}:\n" if $eg->{summary};
-            push @help, "  % $cmdline\n";
-            push @help, "\n" if $eg->{summary} && $i < @$egs;
-        }
-    }
-
-    # description
-    {
-        my $desc = ($scd ? $scd->{description} : undef) //
-            $meta->{description};
-        last unless $desc;
-        push @help, "\n";
-        $desc =~ s/\A\n+//;
-        $desc =~ s/\n+\z//;
-        push @help, $desc, "\n";
-    }
-
-    # options
-    {
-        require Data::Dmp;
-
-        my $opts = $clidocdata->{opts};
-        last unless keys %$opts;
-
-        # find all the categories
-        my %cats; # val=[options...]
-        for (keys %$opts) {
-            push @{ $cats{$opts->{$_}{category}} }, $_;
-        }
-
-        for my $cat (sort keys %cats) {
-            # find the longest option
-            my @opts = sort {length($b)<=>length($a)} @{ $cats{$cat} };
-            my $len = length($opts[0]);
-            # sort again by name
-            @opts = sort {
-                (my $a_without_dash = $a) =~ s/^-+//;
-                (my $b_without_dash = $b) =~ s/^-+//;
-                lc($a) cmp lc($b);
-            } @opts;
-            push @help, "\n$cat:\n";
-            for my $opt (@opts) {
-                my $ospec = $opts->{$opt};
-                my $arg_spec = $ospec->{arg_spec};
-                my $is_bool = $arg_spec->{schema} &&
-                    $arg_spec->{schema}[0] eq 'bool';
-                my $show_default = exists($ospec->{default}) &&
-                    !$is_bool && !$ospec->{is_base64} &&
-                        !$ospec->{is_json} && !$ospec->{is_yaml} &&
-                            !$ospec->{is_alias};
-
-                my $add_sum = '';
-                if ($ospec->{is_base64}) {
-                    $add_sum = " (base64-encoded)";
-                } elsif ($ospec->{is_json}) {
-                    $add_sum = " (JSON-encoded)";
-                } elsif ($ospec->{is_yaml}) {
-                    $add_sum = " (YAML-encoded)";
-                }
-
-                my $argv = '';
-                if (!$ospec->{main_opt} && defined($ospec->{pos})) {
-                    if ($ospec->{greedy}) {
-                        $argv = " (=arg[$ospec->{pos}-])";
-                    } else {
-                        $argv = " (=arg[$ospec->{pos}])";
-                    }
-                }
-
-                my $cmdline_src = '';
-                if (!$ospec->{main_opt} && defined($arg_spec->{cmdline_src})) {
-                    $cmdline_src = " (or from $arg_spec->{cmdline_src})";
-                    $cmdline_src =~ s!_or_!/!g;
-                }
-
-                push @help, sprintf(
-                    "  %-${len}s  %s%s%s%s%s\n",
-                    $opt,
-                    $ospec->{summary}//'',
-                    $add_sum,
-                    $argv,
-                    $cmdline_src,
-                    ($show_default ?
-                         " [".Data::Dmp::dmp($ospec->{default})."]":""),
-
-                );
-            }
-        }
-    }
-
-    [200, "OK", join("", @help), {"cmdline.skip_format"=>1}];
+    $res->[3]{"cmdline.skip_format"} = 1;
+    $res;
 }
 
 sub run_call {
@@ -504,7 +376,7 @@ Perinci::CmdLine::Lite - A lightweight Rinci/Riap-based command-line application
 
 =head1 VERSION
 
-This document describes version 0.70 of Perinci::CmdLine::Lite (from Perl distribution Perinci-CmdLine-Lite), released on 2014-12-20.
+This document describes version 0.71 of Perinci::CmdLine::Lite (from Perl distribution Perinci-CmdLine-Lite), released on 2014-12-23.
 
 =head1 SYNOPSIS
 
